/*============================================================================
  File:     sparse_columns.sql

  Summary:  This script shows how to define and use sparse columns that are 
			supported in SQL Server 2008.

  Date:     August 2008

  SQL Server Version: 10.0.1600.22 (RTM)
------------------------------------------------------------------------------
  Written by Bob Beauchemin, SQLskills.com

  For more scripts and sample code, check out http://www.SQLskills.com

  This script is intended only as a supplement to SQL Server Jumpstart program. 
  
  THIS CODE AND INFORMATION ARE PROVIDED "AS IS" WITHOUT WARRANTY OF 
  ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED 
  TO THE IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
  PARTICULAR PURPOSE.
============================================================================*/

if DATABASEPROPERTY('testdb', 'Version') is not null
drop database testdb
go

create database testdb
go

use testdb
go

-- Create a table with sparse properties and column_set 
CREATE TABLE dbo.products(
	id		INT PRIMARY KEY NOT NULL,
	name	NVARCHAR(100) NOT NULL,
	price	MONEY NULL,
	amount	VARCHAR(50) SPARSE NULL,
	color	VARCHAR(50) SPARSE NULL,
	fiber	VARCHAR(50) SPARSE NULL,
	[len]	VARCHAR(50) SPARSE NULL,
	pitch	VARCHAR(50) SPARSE NULL,
	[type]	VARCHAR(50) SPARSE NULL,
	width	VARCHAR(50) SPARSE NULL,
	col_VALUES XML COLUMN_SET FOR ALL_SPARSE_COLUMNS  NULL,
)
GO

-- insert couple of different products
INSERT INTO dbo.products(id, name, price, pitch, len) 
	VALUES (1, 'Attach-It', 2.00, '12-3', '1 yd');
INSERT INTO dbo.products(id, name, price, color, len, fiber, width) 
	VALUES (2, 'Rug', 149.00, 'white', '1 ft', '1 ft', 'wool')
INSERT INTO dbo.products(id, name, price, color, type, amount) 
	VALUES (3, 'Paint', 12.95, 'white', 'acrylic', '1 qt')
go
	
-- SELECT * from works differently if you have a column_set
-- Compute scalar ConvertSVToXMl
SELECT * FROM dbo.products
	
-- Access individual product properties
SELECT id, pitch, len from dbo.products 
WHERE name='Attach-It';
go

select * from products
go

-- Generic XML access to properties through sparse_column_set
-- Returns an aggregated XML fragment for all non-null sparse columns
SELECT id, name, col_VALUES from dbo.products; 

-- Compute Scalar ConvertXmlToRESV
-- Insert a new product through sparse_column_set. 
DECLARE @xml xml = '<len>.3 in</len><pitch>4-9</pitch>'
INSERT INTO dbo.products (id, name, col_VALUES) 
	      VALUES (4, 'Attach-It', @xml);
go

DECLARE @xml xml = '<len>1.1 ft</len><pitch>12-3</pitch>'
UPDATE dbo.products SET col_VALUES = @xml WHERE id = 1;
go

-- filtered index on sparse columns
CREATE INDEX color_idx ON dbo.products(color) 
WHERE color IS NOT NULL
go

-- index is also covering for this query
SELECT id
FROM dbo.products
WHERE color = 'white'
go

SELECT id, *
FROM dbo.products
WHERE color = 'white'
go

CREATE INDEX rug_fiber_idx ON dbo.products(fiber) 
WHERE name = 'Rug'
go

-- index is also covering for this query
SELECT id, fiber
FROM dbo.products
WHERE name = 'Rug'
AND fiber = 'wool'
go

-- filtered statistics
CREATE STATISTICS fiber_stats ON dbo.products(fiber) 
WHERE fiber IS NOT NULL;                                  
go