
-- hardware store as XML documents

-- non-sparse properties are elements 
-- sparse properties are attributes

-- no namespace schema
CREATE XML SCHEMA COLLECTION dbo.productcol AS
'<xs:schema attributeFormDefault="unqualified" elementFormDefault="qualified" xmlns:xs="http://www.w3.org/2001/XMLSchema">
  <xs:element name="product">
    <xs:complexType>
      <xs:sequence>
        <xs:element name="id" type="xs:integer" />
        <xs:element name="name" type="xs:string" />
        <xs:element name="price" type="xs:decimal" />
        <xs:element name="values">
          <xs:complexType>
              <xs:anyAttribute processContents="skip" /> 
          </xs:complexType>
        </xs:element>
      </xs:sequence>
    </xs:complexType>
  </xs:element>
</xs:schema>'
GO

create function dbo.getid (@data xml(dbo.productcol))
returns int
with schemabinding
as
begin
 return @data.value('(/product/id)[1]', 'int')
end
go

create table productsx (
 productdoc xml(dbo.productcol),
 id as dbo.getid(productdoc) persisted primary key
)
go

insert into productsx values('
<product>
  <id>1</id>
  <name>Attach-It</name>
  <price>2.00</price>
  <values pitch="12-3" len="1 yd"/>
</product>
')
go

insert into productsx values('
<product>
  <id>2</id>
  <name>Rug</name>
  <price>149.00</price>
  <values color="white" len="1 ft" width="1 ft" fiber="wool"/>
</product>
')
go

insert into productsx values('
<product>
  <id>3</id>
  <name>Paint</name>
  <price>12.95</price>
  <values color="blue" type="acrylic" amount="1 gal"/>
</product>
')
go

-- create XML index
CREATE PRIMARY XML INDEX xidx ON productsx(productdoc)
GO
-- XML VALUE INDEX and XML PROPERTY INDEX useful
CREATE XML INDEX xidx_value ON productsx(productdoc)
USING XML INDEX xidx FOR VALUE
GO

-- reconstruct EAV tables
SELECT t.c.value('(./id)[1]', 'int') as id,
       t.c.value('(./name)[1]', 'varchar(30)') as name,
       t.c.value('(./price)[1]', 'money') as price
FROM productsx
CROSS APPLY productdoc.nodes('/product') as t(c)
go

SELECT p.c.value('(./id)[1]', 'int') as id,
	   t.c.value('local-name(.)', 'varchar(30)') as name,
       convert(varchar(30),t.c.query('data(.)')) as value
FROM productsx
CROSS APPLY productdoc.nodes('/product') as p(c)
CROSS APPLY productdoc.nodes('/product/values/@*') as t(c)
go






